@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# pylint: disable-msg=C0301,W0613,W0612,W0212,W0511,R0912,C0322,W0704
+# pylint: disable-msg=W0613,W0612,W0212,W0511,R0912,C0322,W0704
+# W0511 = XXX (my own todo's)
 
 """
 
@@ -27,10 +28,11 @@ import optparse
 import pkg_resources
 import webbrowser
 from distutils.sysconfig import get_python_lib
+import logging
 
 from yolk import __version__
 from yolk.metadata import get_metadata
-from yolk.yolklib import get_highest_installed, get_highest_version, Distributions
+from yolk.yolklib import get_highest_version, Distributions
 from yolk.pypi import CheeseShop
 from yolk.setuptools_support import get_download_uri
 from yolk.plugins import load_plugins
@@ -42,7 +44,7 @@ class Usage(Exception):
 
     def __init__(self, msg):
         """init"""
-        print >> sys.stderr, "%s" % msg
+        LOGGER.error("%s" % msg)
         sys.exit(2)
 
 
@@ -61,14 +63,21 @@ def get_pkglist():
     return pkgs
 
 
-def show_updates(package_name="", version=""):
+def show_updates(package_name=""):
     """Check installed packages for available updates on PyPI"""
     pypi = CheeseShop()
-
     dists = Distributions()
-    for pkg in get_pkglist():
+    if package_name:
+        #Check for a single package
+        pkg_list = []
+        pkg_list.append(package_name)
+    else:
+        #Check for every installed package
+        pkg_list = get_pkglist()
+
+    for pkg in pkg_list:
         for (dist, active) in dists.get_distributions("all", pkg,
-                get_highest_installed(pkg)):
+                dists.get_highest_installed(pkg)):
             (project_name, versions) = \
                     pypi.query_versions_pypi(dist.project_name, True)
             if versions:
@@ -97,8 +106,7 @@ def get_plugin(method, options):
         plugin.configure(options, None)
         if plugin.enabled:
             if not hasattr(plugin, method):
-                print >> sys.stderr, \
-                        "Error: plugin has no method: %s" % method
+                LOGGER.warn("Error: plugin has no method: %s" % method)
                 plugin = None
             else:
                 all_plugins.append(plugin)
@@ -137,7 +145,8 @@ def show_distributions(show, project_name, version, options):
         if metadata:
             add_column_text = ""
             for my_plugin in plugins:
-                #See if package is 'owned' by a package manager such as portage, apt etc.
+                #See if package is 'owned' by a package manager such as
+                #portage, apt, rpm etc.
                 #add_column_text += my_plugin.add_column(filename) + " "
                 add_column_text += my_plugin.add_column(dist) + " "
             print_metadata(metadata, develop, active, options, add_column_text)
@@ -150,7 +159,7 @@ def show_distributions(show, project_name, version, options):
         else:
             pkg_spec = "%s" % project_name
 
-        print >> sys.stderr, "%s is not installed." % pkg_spec
+        LOGGER.error("%s is not installed." % pkg_spec)
         return 2
     elif show == "all" and results and fields:
         print "Versions with '*' are non-active."
@@ -189,7 +198,7 @@ def print_metadata(metadata, develop, active, options, installed_by):
                                 development_status)
     else:
 
-        #XXX Need intelligent justification
+        # Need intelligent justification
 
         print metadata['Name'].ljust(15) + " - " + version.ljust(12) + \
             " - " + status
@@ -230,8 +239,8 @@ You can also specify a package name and version:
     pkgs = pkg_resources.Environment()
 
     if not len(pkgs[project_name]):
-        print >> sys.stderr, "Can't find package for %s" % project_name
-        sys.exit(2)
+        LOGGER.error("Can't find package for %s" % project_name)
+        return 2
 
     for pkg in pkgs[project_name]:
         if not ver:
@@ -248,9 +257,9 @@ You can also specify a package name and version:
                     print "  " + str(pkg._dep_map.values()[0][i - 1])
                 i -= 1
         else:
-            print >> sys.stderr, \
-                "No dependency information was supplied with the package."
-            sys.exit(2)
+            LOGGER.error(\
+                    "No dependency information was supplied with the package.")
+            return 2
 
 
 #PyPI functions
@@ -259,17 +268,15 @@ You can also specify a package name and version:
 
 def show_download_links(package_name, version, file_type):
     """Query PyPI for pkg download URI for a packge"""
-    pypi = CheeseShop()
 
-    #Try Cheese Shop first
     url = None
-    for url in pypi.get_download_urls(package_name, version, file_type):
-        print url
-
-    #Try setuptools if there is no link on Cheese Shop
+    #Use setuptools monkey-patch to grab url
+    for url in get_download_uri(file_type, package_name, version):
+        print "%s" % url
+    #This should never happen because setuptools checks Cheese Shop?
     if not url:
-        url = get_download_uri(file_type, package_name, version)
-        if url:
+        pypi = CheeseShop()
+        for url in pypi.get_download_urls(package_name, version, file_type):
             print url
 
 def browse_website(package_name, browser=None):
@@ -296,7 +303,7 @@ def browse_website(package_name, browser=None):
     print "No homepage URL found."
 
 
-def show_pkg_metadata_pypi(package_name, version):
+def show_pkg_metadata_pypi(package_name, version, fields):
     """Show pkg metadata queried from PyPI"""
     pypi = CheeseShop()
     (pypi_project_name, versions) = \
@@ -307,12 +314,14 @@ def show_pkg_metadata_pypi(package_name, version):
             metadata = pypi.release_data(pypi_project_name, ver)
             if metadata:
                 for key in metadata.keys():
-                    print "%s: %s" % (key, metadata[key])
+                    if not fields or (fields and fields==key):
+                        print "%s: %s" % (key, metadata[key])
         
     else:
-        print >> sys.stderr, "I'm afraid we have no %s at The Cheese Shop. Perhaps a little red Leicester?" % \
-            package_name
-        sys.exit(2)
+        LOGGER.error(\
+                "I'm afraid we have no %s at The Cheese Shop. \
+                \nPerhaps a little red Leicester?" % package_name)
+        return 2
 
 
 def get_all_versions_pypi(package_name, use_cached_pkglist=False):
@@ -321,15 +330,17 @@ def get_all_versions_pypi(package_name, use_cached_pkglist=False):
     (pypi_project_name, versions) = \
             pypi.query_versions_pypi(package_name, use_cached_pkglist)
 
-    #pypi_project_name may != package_name; it returns the name with correct case
+    #pypi_project_name may != package_name
+    #it returns the name with correct case
     #i.e. You give beautifulsoup but PyPI knows it as BeautifulSoup
 
     if versions:
         print_pkg_versions(pypi_project_name, versions)
     else:
-        print >> sys.stderr, "I'm afraid we have no %s at The Cheese Shop. Perhaps a little red Leicester?" % \
-            package_name
-        sys.exit(2)
+        LOGGER.error(\
+                "I'm afraid we have no %s at The Cheese Shop. \
+                \nPerhaps a little red Leicester?" % package_name)
+        return 2
 
 
 def parse_search_spec(spec):
@@ -468,7 +479,8 @@ def show_entry_map(dist):
     try:
         pprinter.pprint(pkg_resources.get_entry_map(dist))
     except pkg_resources.DistributionNotFound:
-        print "Distribution not found: %s" % dist
+        LOGGER.error("Distribution not found: %s" % dist)
+        return 2
 
 def show_entry_points(module):
     """Show entry points for a module"""
@@ -485,7 +497,8 @@ def show_entry_points(module):
         except ImportError:
             pass
     if not found:
-        print "No entry points found for %s" % module
+        LOGGER.error("No entry points found for %s" % module)
+        return 2
 
 def setup_opt_parser():
     """Setup the optparser"""
@@ -500,7 +513,8 @@ def setup_opt_parser():
     opt_parser.add_option("-v", "--verbose", action='store_true', dest=
                           "verbose", default=False, help=
                           "Be more verbose.")
-
+    #pylint: disable-msg=C0301
+    #line too long
     group_local = optparse.OptionGroup(opt_parser,
             "Query installed Python packages",
             "The following options show information about installed Python packages. Activated packages are normal packages on sys.path that can be imported. Non-activated packages need 'pkg_resources.require()' before they can be imported, such as packages installed with 'easy_install --multi-version'. PKG_SPEC can be either a package name or package name and version e.g. Paste==0.9")
@@ -514,16 +528,6 @@ def setup_opt_parser():
                            'List activated packages installed by ' +
                            'setuptools. Use PKG_SPEC to narrow results.')
 
-    group_local.add_option("--entry-points", action='store',
-                           dest="entry_points", default=False, help=
-                           'List entry points for a module. e.g. --entry-points yolk.plugins',
-                            metavar="MODULE")
-
-    group_local.add_option("--entry-map", action='store',
-                           dest="entry_map", default=False, help=
-                           'List entry map for a distribution. e.g. --entry-map yolk',
-                           metavar="PACKAGE_NAME")
-
     group_local.add_option("-n", "--non-activated", action='store_true',
                            dest="nonactive", default=False, help=
                            'List non-activated packages installed by ' +
@@ -537,13 +541,22 @@ def setup_opt_parser():
     group_local.add_option("-f", "--fields", action="store", dest=
                            "fields", default=False, help=
                            'Show specific metadata fields. ' +
-                           '(use with -l -a or -n)')
+                           '(use with -m or -M)')
 
     group_local.add_option("-d", "--depends", action='store_true', dest=
                            "depends", default=False, help=
                            "Show dependencies for a package installed by " +
                            "setuptools if they are available. (Use with PKG_SPEC)")
 
+    group_local.add_option("--entry-points", action='store',
+                           dest="entry_points", default=False, help=
+                           'List entry points for a module. e.g. --entry-points yolk.plugins',
+                            metavar="MODULE")
+
+    group_local.add_option("--entry-map", action='store',
+                           dest="entry_map", default=False, help=
+                           'List entry map for a distribution. e.g. --entry-map yolk',
+                           metavar="PACKAGE_NAME")
     group_pypi = optparse.OptionGroup(opt_parser,
             "PyPI (Cheese Shop) options",
             "The following options query the Python Package Index:")
@@ -560,12 +573,12 @@ def setup_opt_parser():
 
     group_pypi.add_option("-L", "--latest", action='store_true', dest=
                           "rss_feed", default=False, help=
-                          "Show last 20 updates on PyPI.")
+                          "Show last 20 releases on PyPI.")
 
     group_pypi.add_option("-M", "--query-metadata", action='store_true',
                           dest="query_metadata_pypi", default=False,
                           metavar="PKG_SPEC", help=
-                          "Show metadata for a package listed on PyPI. (Use with PKG_SPEC)")
+                          "Show metadata for a package listed on PyPI. Use -f to show particular fields. (Use with PKG_SPEC)")
 
     group_pypi.add_option("-S", "", action="store", dest="search",
                           default=False, help=
@@ -609,65 +622,69 @@ def main():
     if not options.search and (len(sys.argv) == 1 or len(remaining_args) >
                                2):
         opt_parser.print_help()
-        sys.exit(2)
+        return 2
 
     if options.entry_points:
-        show_entry_points(options.entry_points)
-        sys.exit()
+        return show_entry_points(options.entry_points)
     if options.entry_map:
-        show_entry_map(options.entry_map)
-        sys.exit()
+        return show_entry_map(options.entry_map)
 
     #Options that depend on querying installed packages, not PyPI.
     #We find the proper case for package names if they are installed,
     #otherwise PyPI returns the correct case.
-    if options.depends or options.all or options.active or options.nonactive:
+    if options.depends or options.all or options.active or options.nonactive \
+            or (options.show_updates and remaining_args):
         want_installed = True
     else:
         want_installed = False
     if remaining_args:
         (package, version) = parse_pkg_ver(remaining_args, want_installed)
         if want_installed and not package:
-            print >> sys.stderr, "%s is not installed." % remaining_args[0]
-            sys.exit(2)
+            LOGGER.error("%s is not installed." % remaining_args[0])
+            return 2
     else:
         package = version = None
 
     if options.search:
-        pypi_search(options.search, remaining_args)
+        return pypi_search(options.search, remaining_args)
     elif options.version:
         print "Version %s" % __version__.VERSION
+        return
     elif options.depends:
-        show_deps(remaining_args)
+        return show_deps(remaining_args)
     elif options.all:
         if options.active or options.nonactive:
-            opt_parser.error("Choose either -l, -n or -a, not combinations of those.")
-        show_distributions("all", package, version, options)
+            opt_parser.error("Choose either -l, -n or -a")
+        return show_distributions("all", package, version, options)
     elif options.active:
         if options.all or options.nonactive:
-            opt_parser.error("Choose either -l, -n or -a, not combinations of those.")
-
-        show_distributions("active", package, version, options)
+            opt_parser.error("Choose either -l, -n or -a")
+        return show_distributions("active", package, version, options)
     elif options.nonactive:
         if options.active or options.all:
-            opt_parser.error("Choose either -l, -n or -a, not combinations of those.")
-        show_distributions("nonactive", package, version, options)
+            opt_parser.error("Choose either -l, -n or -a")
+        return show_distributions("nonactive", package, version, options)
     elif options.versions_available:
-        get_all_versions_pypi(package, False)
+        return get_all_versions_pypi(package, False)
     elif options.browse_website:
-        browse_website(package)
+        return browse_website(package)
     elif options.download_links:
-        show_download_links(package, version, options.file_type)
+        return show_download_links(package, version, options.file_type)
     elif options.rss_feed:
-        get_rss_feed()
+        return get_rss_feed()
     elif options.show_updates:
-        show_updates(package, version)
+        return show_updates(package)
     elif options.query_metadata_pypi:
-        show_pkg_metadata_pypi(package, version)
+        return show_pkg_metadata_pypi(package, version, options.fields)
     else:
         opt_parser.print_help()
         return 2
 
+LOGGER = logging.getLogger("yolk")
+LOGGER.setLevel(logging.DEBUG)
+LOGGER.addHandler(logging.StreamHandler())
+LOGGER.setLevel(logging.DEBUG)
+
 if __name__ == "__main__":
     sys.exit(main())
 
