@@ -185,15 +185,18 @@ class CheeseShop:
             if metadata and metadata.has_key('download_url') and \
                         metadata['download_url'] != "UNKNOWN" and \
                         metadata['download_url'] != None:
-                    if metadata['download_url'] not in all_urls:
-                        if pkg_type != "all":
-                            url = filter_url(pkg_type, metadata['download_url'])
-                            if url:
-                                all_urls.append(url)
+                if metadata['download_url'] not in all_urls:
+                    if pkg_type != "all":
+                        url = filter_url(pkg_type, metadata['download_url'])
+                        if url:
+                            all_urls.append(url)
         return all_urls
         
 def filter_url(pkg_type, url):
     """Returns URL of specified file type, else None"""
+    #Remove Source Forge cruft
+    if "?modtime" in url:
+        url = url.split("?")[0]
     #Remove MD5 checksum
     if "#md5=" in url:
         url = url.split("#")[0]
